package com.warewms.ams.ndc.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.warewms.ams.ndc.domain.AmsTask;
import com.warewms.ams.ndc.dto.AmsTaskStateFeedbackDTO;
import com.warewms.ams.ndc.service.IAmsTaskService;
import com.warewms.ams.ndc.service.StatusUpdateService;
import com.warewms.common.constant.WmsConstant;
import com.warewms.common.utils.string.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.retry.annotation.Backoff;
import org.springframework.retry.annotation.Retryable;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

@Slf4j
@Component
public class StatusUpdateServiceImpl implements StatusUpdateService {

    @Autowired
    private IAmsTaskService amsTaskService;

    private RestTemplate restTemplate = new RestTemplate();

    private final String WMS_URL = "localhost:28080/erp/execResult";

    @Retryable(
            maxAttempts = 4,
            backoff = @Backoff(delay = 2000, multiplier = 2)
    )
    @Override
    public void taskStateResponse(Integer state, Integer carNo, Integer aciIndex) {
        AmsTask amsTask = amsTaskService.getAmsTaskByAciIndex(aciIndex);
        if (ObjectUtil.isNull(amsTask)) return;
        if (StringUtils.isNotBlank(amsTask.getExt3()) &&
                (state.equals(WmsConstant.TASK_STS.TASK2.getValue()) ||
                        state.equals(WmsConstant.TASK_STS.TASK6.getValue()))) {
            return;
        }
        AmsTaskStateFeedbackDTO amsTaskStateFeedbackDTO = new AmsTaskStateFeedbackDTO();
        amsTaskStateFeedbackDTO.setState(state);
        amsTaskStateFeedbackDTO.setTaskNo(String.valueOf(amsTask.getId()));
        amsTaskStateFeedbackDTO.setBusinessNo(amsTask.getTaskNo());
        amsTaskStateFeedbackDTO.setCarNo(carNo);
        Object response = null;
        try {
            response = restTemplate.postForObject(WMS_URL, amsTaskStateFeedbackDTO, Object.class);
        } catch (RestClientException e) {
            log.error("AMS send request to WMS is on error, pause is :{}", e.getMessage());
        }
        log.info("wms response data is :{}", JSONObject.toJSONString(response));
    }

    @Override
    public void updateStatus(Long id, Integer carNo) {
        AmsTask amsTask = amsTaskService.selectAmsTaskById(id);
        if (ObjectUtil.isNull(amsTask)) return;

        log.info("任务回调-------------------------》" + amsTask.getTaskNo());
        if (amsTask.getAciIndex() == null) {
            log.error("没有index，未下发的任务");
        }
        amsTask.setSystemStatus(WmsConstant.TASK_STS.getValueByKey(amsTask.getEvent()));
        amsTaskService.updateAmsTask(amsTask);
        if (StringUtils.isNotBlank(amsTask.getExt3()) &&
                (amsTask.getSystemStatus().equals(WmsConstant.TASK_STS.TASK2.getValue()) ||
                        amsTask.getSystemStatus().equals(WmsConstant.TASK_STS.TASK6.getValue()))) {
            amsTaskService.ndcAmsTaskInsert(amsTask.getTaskNo(), amsTask.getBusinessType(), amsTask.getStTo(), Integer.parseInt(amsTask.getExt3()));
        }
    }
}
